/*
 * Created on 01.10.2004 by HS
 * 
 */
package de.laliluna.tutorial.library.ejb;


import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import de.laliluna.tutorial.library.interfaces.BookLocal;
import de.laliluna.tutorial.library.interfaces.BookLocalHome;
import de.laliluna.tutorial.library.interfaces.ClientLocal;
import de.laliluna.tutorial.library.interfaces.ClientLocalHome;


/**
 * @author HS
 *
 * 
 */

/**
 * XDoclet-based stateless session bean.  The class must be declared
 * <code>public</code>, according to the EJB specification.<br>
 * 
 * To generate code:
 * <br>
 * <ul>
 * <li> Add Standard EJB module to XDoclet project properties
 * <li> Customize XDoclet configuration
 * <li> Run XDoclet
 * </ul>
 * <br>
 * Please see the included XDoclet Overview 
 * and the XDoclet Reference in the help system for details
 * 
 * @ejb.bean name = "LibraryManager"
 *           type = "Stateless"
 *           display-name = "LibraryManager"
 *           description = "LibraryManager EJB"
 *           view-type = "remote"
 *           jndi-name = "ejb/LibraryManagerHome"
 */
public abstract class LibraryManager implements SessionBean {

  /** The SessionContext */
  private SessionContext context;

  
  /**
   * lend a book by client and sets availability to false
   * @ejb.interface-method view-type = "remote"
   * @param clientId
   * @param bookId
   * @throws EJBException
   */
  public void lendBook(String clientId, String bookId) throws EJBException{
    try {
      InitialContext initialContext = new InitialContext();
      
      // [laliluna] 01.10.2004 get homeinterface by JNDI lookup and find client by id
      ClientLocalHome clientLocalHome = (ClientLocalHome) initialContext.lookup(ClientLocalHome.JNDI_NAME);
      ClientLocal clientLocal = clientLocalHome.findByPrimaryKey(clientId);
      
      // [laliluna] 01.10.2004 get homeinterface by JNDI lookup and find book by id
      BookLocalHome bookLocalHome = (BookLocalHome) initialContext.lookup(BookLocalHome.JNDI_NAME);
      BookLocal bookLocal = bookLocalHome.findByPrimaryKey(bookId);
      
      if (!bookLocal.getAvailable())
        System.out.println("Book is not available");
      else
      {
      bookLocal.setClientLocal(clientLocal);
      bookLocal.setAvailable(false);
      }
      
      /*
       * [laliluna] 01.10.2004
       * alternative you may use
       * clientLocal.getBook().add(bookLocal); 
       */
      
    } catch (NamingException e) {
      e.printStackTrace();
    } catch (FinderException e) {
      e.printStackTrace();
    }
    
    
  }
  
  /**
   * returns a book from a client and sets availability to true
   * @ejb.interface-method view-type = "remote"
   * @param clientId
   * @param bookId
   * @throws EJBException
   */
  public void returnBook(String clientId, String bookId) throws EJBException{
    try {
      InitialContext initialContext = new InitialContext();
      
      // [laliluna] 01.10.2004 get homeinterface by JNDI lookup and find client by id
      ClientLocalHome clientLocalHome = (ClientLocalHome) initialContext.lookup(ClientLocalHome.JNDI_NAME);
      ClientLocal clientLocal = clientLocalHome.findByPrimaryKey(clientId);
      
      // [laliluna] 01.10.2004 get homeinterface by JNDI lookup and find book by id
      BookLocalHome bookLocalHome = (BookLocalHome) initialContext.lookup(BookLocalHome.JNDI_NAME);
      BookLocal bookLocal = bookLocalHome.findByPrimaryKey(bookId);
      
      bookLocal.setAvailable(true);
      bookLocal.setClientLocal(null);
    } catch (NamingException e) {
      e.printStackTrace();
    } catch (FinderException e) {
      e.printStackTrace();
    }
  }
  
  /**
   * An <code>ejbCreate</code> method as required by the EJB specification. <br>
   * 
   * The container calls the instances <code>ejbCreate</code> method whose 
   * signature matches the signature of the <code>create</code> method invoked 
   * by the client. The input parameters sent from the client are passed to
   * the <code>ejbCreate</code> method. Each session bean class must have at
   * least one <code>ejbCreate</code> method. The number and signatures
   * of a session beans <code>create</code> methods are specific to each 
   * session bean class.
   * 
   * @throws CreateException Thrown if the instance could not perform 
   * the function requested by the container because of an system-level error.
   * 
   * @ejb.create-method
   */
  public void ejbCreate() throws CreateException {
  }

  /**
   * The <code>ejbActivate()</code> method as required by the EJB specification.<br>
   *
   * The activate method is called when the instance is activated from its 
   * passive" state. The instance should acquire any resource that it has 
   * released earlier in the <code>ejbPassivate()</code> method. <br>
   * 
   * This method is called with no transaction context. 
   * 
   * @throws EJBException Thrown if the instance could not perform 
   * the function requested by the container because of an system-level error.
   */
  public void ejbActivate() throws EJBException {
  }

  /**
   * The <code>ejbPassivate()</code> method as required by the EJB specification.<br>
   *
   * The activate method is called when the instance is activated from 
   * its "passive" state. The instance should acquire any resource that 
   * it has released earlier in the <code>ejbActivate()</code> method.  <br>
   * 
   * This method is called with no transaction context. 
   * 
   * @throws EJBException Thrown if the instance could not perform 
   * the function requested by the container because of an system-level error.
   */
  public void ejbPassivate() throws EJBException {
  }

  /**
   * The <code>ejbRemove()</code> method as required by the EJB specification.<br>
   *
   * A container invokes this method before it ends the life of the 
   * session object. This happens as a result of a client's invoking 
   * a remove operation, or when a container decides to terminate the 
   * session object after a timeout.   <br>
   * 
   * This method is called with no transaction context. 
   * 
   * @throws EJBException Thrown if the instance could not perform 
   * the function requested by the container because of an system-level error.
   */
  public void ejbRemove() throws EJBException {
  }

  /**
   * Set the associated session context. The container calls this method 
   * after the instance creation.  <br>
   * 
   * The enterprise bean instance should store the reference to the context 
   * object in an instance variable. <br>
   * 
   * This method is called with no transaction context. 
   * 
   * @throws EJBException Thrown if the instance could not perform 
   * the function requested by the container because of an system-level error.
   */
  public void setSessionContext(SessionContext newContext) throws EJBException {
    context = newContext;
  }

}
